@@ -1,22 +1,21 @@
 import json
-from dataclasses import dataclass
-from pathlib import Path
-
 import pytest
 import typer
+from dataclasses import dataclass
+from pathlib import Path
 from typing import Optional, List
 from jinja2 import Environment, PackageLoader, select_autoescape
 from rich.console import Console
 from rich.table import Table
-
-from amora.compilation import amora_model_for_path
+from rich.text import Text
 
 from amora.config import settings
 from amora.models import (
-    list_model_files,
     AmoraModel,
-    list_target_files,
+    list_models,
+    Model,
 )
+from amora.utils import list_target_files
 from amora.compilation import compile_statement
 from amora import materialization
 from amora.providers.bigquery import (
@@ -24,6 +23,8 @@ from amora.providers.bigquery import (
     get_schema,
     BIGQUERY_TYPES_TO_PYTHON_TYPES,
     DryRunResult,
+    estimated_query_cost_in_usd,
+    estimated_storage_cost_in_usd,
 )
 
 app = typer.Typer(
@@ -55,29 +56,22 @@ def compile(
     """
     Generates executable SQL from model files. Compiled SQL files are written to the `./target` directory.
     """
-    for model_file_path in list_model_files():
+    for model, model_file_path in list_models():
         if models and model_file_path.stem not in models:
             continue
 
-        try:
-            AmoraModel_class = amora_model_for_path(model_file_path)
-        except ValueError:
-            continue
-
-        if not issubclass(AmoraModel_class, AmoraModel):  # type: ignore
-            continue
-
-        source_sql_statement = AmoraModel_class.source()
+        source_sql_statement = model.source()
         if source_sql_statement is None:
             typer.echo(f"⏭ Skipping compilation of model `{model_file_path}`")
             continue
 
-        target_file_path = AmoraModel_class.target_path(model_file_path)
+        target_file_path = model.target_path(model_file_path)
         typer.echo(
             f"🏗 Compiling model `{model_file_path}` -> `{target_file_path}`"
         )
 
         content = compile_statement(source_sql_statement)
+        target_file_path.parent.mkdir(parents=True, exist_ok=True)
         target_file_path.write_text(content)
 
 
@@ -88,7 +82,8 @@ def materialize(
     draw_dag: bool = typer.Option(False, "--draw-dag"),
 ) -> None:
     """
-    Executes the compiled SQL againts the current target database.
+    Executes the compiled SQL against the current target database.
+
     """
     model_to_task = {}
 
@@ -97,7 +92,7 @@ def materialize(
             continue
 
         task = materialization.Task.for_target(target_file_path)
-        model_to_task[task.model.__name__] = task
+        model_to_task[task.model.unique_name] = task
 
     dag = materialization.DependencyDAG.from_tasks(tasks=model_to_task.values())
 
@@ -130,7 +125,7 @@ def test(
     Runs tests on data in deployed models. Run this after `amora materialize`
     to ensure that the date state is up-to-date.
     """
-    return_code = pytest.main(["-n", "auto"])
+    return_code = pytest.main(["-n", "auto", "--verbose"])
     raise typer.Exit(return_code)
 
 
@@ -154,11 +149,21 @@ def models_list(
     List the models in your project as a human readable table
     or as a JSON serialized document
 
+    ```shell
+    amora models list
+    ```
+    You can also use the option `--with-total-bytes` to use
+    BigQuery query dry run feature to gather model total bytes information
+
+    ```shell
+    amora models list --with-total-bytes
+    ```
+
     """
 
     @dataclass
     class ResultItem:
-        model: AmoraModel
+        model: Model
         dry_run_result: Optional[DryRunResult] = None
 
         def as_dict(self):
@@ -169,6 +174,8 @@ def models_list(
                 "model_name": self.model_name,
                 "referenced_tables": self.referenced_tables,
                 "total_bytes": self.total_bytes,
+                "estimated_query_cost_in_usd": self.estimated_query_cost_in_usd,
+                "estimated_storage_cost_in_usd": self.estimated_storage_cost_in_usd,
             }
 
         @property
@@ -188,6 +195,24 @@ def models_list(
                 ]
             )
 
+        @property
+        def estimated_query_cost_in_usd(self) -> Optional[str]:
+            if self.dry_run_result:
+                cost = estimated_query_cost_in_usd(
+                    self.dry_run_result.total_bytes
+                )
+                return f"{cost:.{settings.MONEY_DECIMAL_PLACES}f}"
+            return None
+
+        @property
+        def estimated_storage_cost_in_usd(self) -> Optional[str]:
+            if self.dry_run_result:
+                cost = estimated_storage_cost_in_usd(
+                    self.dry_run_result.total_bytes
+                )
+                return f"{cost:.{settings.MONEY_DECIMAL_PLACES}f}"
+            return None
+
         @property
         def total_bytes(self) -> Optional[int]:
             if self.dry_run_result:
@@ -208,8 +233,9 @@ def models_list(
                 return None
 
     results = []
-    for model_file_path in list_model_files():
-        model = amora_model_for_path(model_file_path)
+    placeholder = "-"
+
+    for model, model_file_path in list_models():
         if with_total_bytes:
             result_item = ResultItem(model=model, dry_run_result=dry_run(model))
         else:
@@ -228,19 +254,28 @@ def models_list(
 
         table.add_column("Model name", style="green bold", no_wrap=True)
         table.add_column("Total bytes", no_wrap=True)
-        table.add_column("Referenced tables", no_wrap=True)
-        table.add_column("Depends on", no_wrap=True)
+        table.add_column("Estimated query cost", no_wrap=True)
+        table.add_column("Estimated storage cost", no_wrap=True)
+        table.add_column("Referenced tables")
+        table.add_column("Depends on")
         table.add_column("Has source?", no_wrap=True, justify="center")
         table.add_column("Materialization", no_wrap=True)
 
         for result in results:
             table.add_row(
                 result.model_name,
-                f"{result.total_bytes or '-'}",
-                " , ".join(result.referenced_tables) or "-",
-                " , ".join(result.depends_on) or "-",
+                f"{result.total_bytes or placeholder}",
+                result.estimated_query_cost_in_usd or placeholder,
+                result.estimated_storage_cost_in_usd or placeholder,
+                Text(
+                    "\n".join(result.referenced_tables) or placeholder,
+                    overflow="fold",
+                ),
+                Text(
+                    "\n".join(result.depends_on) or placeholder, overflow="fold"
+                ),
                 "🟢" if result.has_source else "🔴",
-                result.materialization_type or "-",
+                result.materialization_type or placeholder,
             )
 
         console = Console(width=settings.CLI_CONSOLE_MAX_WIDTH)
@@ -273,7 +308,7 @@ def models_import(
     Generates a new amora model file from an existing table/view
 
     ```shell
-    $ amora models import --table-reference my_gcp_project.my_dataset.my_table my_gcp_project/my_dataset/my_table
+    amora models import --table-reference my_gcp_project.my_dataset.my_table my_gcp_project/my_dataset/my_table
     ```
     """
 
